#include "init.h"	// included by C-Wiz
#include <htc.h>

/* RB4 - SETTING_STROBE	o
 * RB5 - BUS_IN_USE		io
 * RB6 - USB RD			o
 * RB7 - USB RXF			i
 * use RA4&5 for RS pins if required
 * connect PWREN to RA2? 
 * PORTC is data bus			*/
#define SETTING_STROBE 	RB4
#define BUS_IN_USE	 	RB5
#define USB_RD 			RB6
#define USB_RXF 			RB7
#define TEST_LED			RA2
#define PAL_RS0			RA4
#define PAL_RS1			RA5		//settings register selects in CPLD
#define BUS_IN_USE_MASK 	0b00100000

typedef char byte;
// prototype
void WriteSettings(byte, byte);
void CheckCommand(byte cmmd, byte value);

// Global variables
byte status, level, holdoff, clkdiv;

void
main(void)
{
	byte i, command, value;
/*	status = 0b00001011;
	level = 0x7f;
	holdoff = 0x7f;
	clkdiv = 0x00;*/
	
	init();	// Function call inserted by C-Wiz
	
	//AN ports must be un-selected before use as digital IO
	ANSEL = 0;
	ANSELH = 0;
	
	USB_RD = 1; //initially take USB_RD high
	SETTING_STROBE = 0; // SETTING STROBE initially 0
	TEST_LED = 0; //Clear test LED
	
	//WriteSettings(status, level, holdoff, clkdiv);  //***** NO BIU CHECK OR ASSERTION!
	
	while (1){
		if (!USB_RXF)
		{
			//Data to be read from USB FIFO
			while (BUS_IN_USE) {}; //wait for BUS-IN-USE to go low
			TRISB &= ~BUS_IN_USE_MASK; //Make BIU output
			BUS_IN_USE = 1; //Assert BUS-IN-USE
			
			// Read command in from USB
			USB_RD = 0;
			//for (i = 50; i>0; i--){}
			command = PORTC;
			USB_RD = 1;
			
			if ((command & 0xF0) == 0x50)
			{
				TEST_LED = 1; //light test LED
				if (command == 0x55){
					value = 0x80; //55 is DTR ready command, set bit seven to pass clock pulse through
				}
				else {
					while (USB_RXF) {}; //wait for RXFlag to go low again
					
					// Read value in from USB
					USB_RD = 0;
					value = PORTC;
					USB_RD = 1;
				}
				
				CheckCommand(command, value);
				
				TEST_LED = 0; //clear test LED
			}
			
			TRISB |= BUS_IN_USE_MASK; //Un-assert bus-in-use
			BUS_IN_USE = 0;
		}
	}
}

void WriteSettings(byte rs, byte value)
{
	// This assumes that BIU has been checked and asserted

	// Set PORTC as outputs
	TRISC = 0x00;
	
	PAL_RS0 = rs & 0b01;
	PAL_RS1 = (rs & 0b10) >> 1;
	PORTC = value;
	SETTING_STROBE = 1;
	SETTING_STROBE = 0; // STROBE the CPLD
	
	// Set PORTC back to inputs
	TRISC = 0xFF;
	
	PAL_RS0 = 0;
	PAL_RS1 = 0; //clear the rs pins, stops clock setting being re-altered
}

void CheckCommand(byte cmmd, byte value)
{
	// Put that setting into correct variable according to command
			switch (cmmd)
			{
				case 0x50: 	
				case 0x55:	WriteSettings(0b00,value);
							break;
				case 0x51:	WriteSettings(0b01,value);
							break;
				case 0x52:	WriteSettings(0b10,value);
							break;
				case 0x53:	WriteSettings(0b11,value);
							break;
				default:		return; // otherwise don't reload settings
			}
			
}
